@@ -43,6 +43,8 @@ private_profile = None
 # The global WebEngineSettings object
 global_settings = None
 
+default_user_agent = None
+
 
 class _SettingsWrapper:
 
@@ -52,8 +54,9 @@ class _SettingsWrapper:
     """
 
     def __init__(self):
-        self._settings = [default_profile.settings(),
-                          private_profile.settings()]
+        self._settings = [default_profile.settings()]
+        if private_profile:
+            self._settings.append(private_profile.settings())
 
     def setAttribute(self, *args, **kwargs):
         for settings in self._settings:
@@ -193,10 +196,17 @@ class ProfileSetter:
         """Initialize settings on the given profile."""
         self.set_http_headers()
         self.set_http_cache_size()
+        self._set_hardcoded_settings()
+        if qtutils.version_check('5.8'):
+            self.set_dictionary_language()
 
+    def _set_hardcoded_settings(self):
+        """Set up settings with a fixed value."""
         settings = self._profile.settings()
+
         settings.setAttribute(
             QWebEngineSettings.FullScreenSupportEnabled, True)
+
         try:
             settings.setAttribute(
                 QWebEngineSettings.FocusOnNavigationEnabled, False)
@@ -204,8 +214,11 @@ class ProfileSetter:
             # Added in Qt 5.8
             pass
 
-        if qtutils.version_check('5.8'):
-            self.set_dictionary_language()
+        try:
+            settings.setAttribute(QWebEngineSettings.PdfViewerEnabled, False)
+        except AttributeError:
+            # Added in Qt 5.13
+            pass
 
     def set_http_headers(self):
         """Set the user agent and accept-language for the given profile.
@@ -265,10 +278,12 @@ def _update_settings(option):
     if option in ['content.headers.user_agent',
                   'content.headers.accept_language']:
         default_profile.setter.set_http_headers()
-        private_profile.setter.set_http_headers()
+        if private_profile:
+            private_profile.setter.set_http_headers()
     elif option == 'content.cache.size':
         default_profile.setter.set_http_cache_size()
-        private_profile.setter.set_http_cache_size()
+        if private_profile:
+            private_profile.setter.set_http_cache_size()
     elif (option == 'content.cookies.store' and
           # https://bugreports.qt.io/browse/QTBUG-58650
           qtutils.version_check('5.9', compiled=False)):
@@ -276,14 +291,16 @@ def _update_settings(option):
         # We're not touching the private profile's cookie policy.
     elif option == 'spellcheck.languages':
         default_profile.setter.set_dictionary_language()
-        private_profile.setter.set_dictionary_language(warn=False)
+        if private_profile:
+            private_profile.setter.set_dictionary_language(warn=False)
 
 
 def _init_profiles():
     """Init the two used QWebEngineProfiles."""
-    global default_profile, private_profile
+    global default_profile, private_profile, default_user_agent
 
     default_profile = QWebEngineProfile.defaultProfile()
+    default_user_agent = default_profile.httpUserAgent()
     default_profile.setter = ProfileSetter(default_profile)
     default_profile.setCachePath(
         os.path.join(standarddir.cache(), 'webengine'))
@@ -292,10 +309,11 @@ def _init_profiles():
     default_profile.setter.init_profile()
     default_profile.setter.set_persistent_cookie_policy()
 
-    private_profile = QWebEngineProfile()
-    private_profile.setter = ProfileSetter(private_profile)
-    assert private_profile.isOffTheRecord()
-    private_profile.setter.init_profile()
+    if not qtutils.is_single_process():
+        private_profile = QWebEngineProfile()
+        private_profile.setter = ProfileSetter(private_profile)
+        assert private_profile.isOffTheRecord()
+        private_profile.setter.init_profile()
 
 
 def init(args):
