@@ -402,7 +402,8 @@ class Command:
         if isinstance(typ, tuple):
             raise TypeError("{}: Legacy tuple type annotation!".format(
                 self.name))
-        elif getattr(typ, '__origin__', None) is typing.Union or (
+
+        if getattr(typ, '__origin__', None) is typing.Union or (
                 # Older Python 3.5 patch versions
                 # pylint: disable=no-member,useless-suppression
                 hasattr(typing, 'UnionMeta') and
