@@ -28,6 +28,7 @@ import traceback
 import configparser
 import contextlib
 import typing
+import re
 
 import yaml
 from PyQt5.QtCore import pyqtSignal, QObject, QSettings, qVersion
@@ -270,6 +271,16 @@ class YamlConfig(QObject):
                     settings[name][scope] = true_value if val else false_value
                     self._mark_changed()
 
+    def _migrate_string_value(self, settings: _SettingsType, name: str,
+                              source: str, target: str) -> None:
+        if name in settings:
+            for scope, val in settings[name].items():
+                if isinstance(val, str):
+                    new_val = re.sub(source, target, val)
+                    if new_val != val:
+                        settings[name][scope] = new_val
+                        self._mark_changed()
+
     def _handle_migrations(self, settings: _SettingsType) -> '_SettingsType':
         """Migrate older configs to the newest format."""
         # Simple renamed/deleted options
@@ -326,6 +337,12 @@ class YamlConfig(QObject):
         self._migrate_bool(settings, 'qt.force_software_rendering',
                            'software-opengl', 'none')
 
+        for s in ['tabs.title.format',
+                  'tabs.title.format_pinned',
+                  'window.title_format']:
+            self._migrate_string_value(
+                settings, s, r'(?<!{)\{title\}(?!})', r'{current_title}')
+
         return settings
 
     def _validate(self, settings: _SettingsType) -> None:
