@@ -54,6 +54,11 @@ from qutebrowser.utils import log, utils, standarddir, usertypes, message
 from qutebrowser.misc import objects, earlyinit, sql, httpclient, pastebin
 from qutebrowser.browser import pdfjs
 
+try:
+    from qutebrowser.browser.webengine import webenginesettings
+except ImportError:  # pragma: no cover
+    webenginesettings = None  # type: ignore
+
 
 @attr.s
 class DistributionInfo:
@@ -69,7 +74,8 @@ class DistributionInfo:
 pastebin_url = None
 Distribution = enum.Enum(
     'Distribution', ['unknown', 'ubuntu', 'debian', 'void', 'arch',
-                     'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro'])
+                     'gentoo', 'fedora', 'opensuse', 'linuxmint', 'manjaro',
+                     'kde'])
 
 
 def distribution():
@@ -94,9 +100,9 @@ def distribution():
     except (OSError, UnicodeDecodeError):
         return None
 
-    pretty = info.get('PRETTY_NAME', 'Unknown')
-    if pretty == 'Linux':  # Thanks, Funtoo
-        pretty = info.get('NAME', pretty)
+    pretty = info.get('PRETTY_NAME', None)
+    if pretty in ['Linux', None]:  # Funtoo has PRETTY_NAME=Linux
+        pretty = info.get('NAME', 'Unknown')
 
     if 'VERSION_ID' in info:
         dist_version = pkg_resources.parse_version(info['VERSION_ID'])
@@ -106,6 +112,7 @@ def distribution():
     dist_id = info.get('ID', None)
     id_mappings = {
         'funtoo': 'gentoo',  # does not have ID_LIKE=gentoo
+        'org.kde.Platform': 'kde',
     }
     try:
         parsed = Distribution[id_mappings.get(dist_id, dist_id)]
@@ -324,7 +331,7 @@ def _chromium_version():
 
     Qt 5.9:  Chromium 56
     (LTS)    56.0.2924.122 (2017-01-25)
-             5.9.7: Security fixes up to 69.0.3497.113 (2018-09-27)
+             5.9.8: Security fixes up to 72.0.3626.121 (2019-03-01)
 
     Qt 5.10: Chromium 61
              61.0.3163.140 (2017-09-05)
@@ -336,19 +343,22 @@ def _chromium_version():
 
     Qt 5.12: Chromium 69
     (LTS)    69.0.3497.113 (2018-09-27)
-             5.12.1: Security fixes up to 71.0.3578.94 (2018-12-14)
-             5.12.2: Security fixes up to 72.0.3626.96 (2019-02-06)
+             5.12.4: Security fixes up to 74.0.3729.157 (2019-05-14)
 
-    Qt 5.13: (in development) Chromium 71 merged, 73 in review.
+    Qt 5.13: Chromium 73
+             73.0.3683.105 (~2019-02-28)
+             5.13.0: Security fixes up to 74.0.3729.131 (2019-04-30)
 
     Also see https://www.chromium.org/developers/calendar
     and https://chromereleases.googleblog.com/
     """
-    if QWebEngineProfile is None:
+    if webenginesettings is None or QWebEngineProfile is None:
         # This should never happen
         return 'unavailable'
-    profile = QWebEngineProfile()
-    ua = profile.httpUserAgent()
+    ua = webenginesettings.default_user_agent
+    if ua is None:
+        profile = QWebEngineProfile.defaultProfile()
+        ua = profile.httpUserAgent()
     match = re.search(r' Chrome/([^ ]*) ', ua)
     if not match:
         log.misc.error("Could not get Chromium version from: {}".format(ua))
